'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = attachScopes;

var _estreeWalker = require('estree-walker');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var blockDeclarations = {
	'const': true,
	'let': true
};

var extractors = {
	Literal: function Literal(names, param) {
		names.push(param.value);
	},
	Identifier: function Identifier(names, param) {
		names.push(param.name);
	},
	ObjectPattern: function ObjectPattern(names, param) {
		param.properties.forEach(function (prop) {
			if (prop.type === 'RestElement') {
				extractors.RestElement(names, prop);
			} else {
				extractors[(prop.value || prop.key).type](names, prop.value || prop.key);
			}
		});
	},
	ArrayPattern: function ArrayPattern(names, param) {
		param.elements.forEach(function (element) {
			if (element) extractors[element.type](names, element);
		});
	},
	RestElement: function RestElement(names, param) {
		extractors[param.argument.type](names, param.argument);
	},
	AssignmentPattern: function AssignmentPattern(names, param) {
		return extractors[param.left.type](names, param.left);
	}
};

function extractNames(param) {
	var names = [];

	extractors[param.type](names, param);
	return names;
}

var Scope = function () {
	function Scope(options) {
		var _this = this;

		_classCallCheck(this, Scope);

		options = options || {};

		this.parent = options.parent;
		this.isBlockScope = !!options.block;

		this.declarations = Object.create(null);

		if (options.params) {
			options.params.forEach(function (param) {
				extractNames(param).forEach(function (name) {
					_this.declarations[name] = true;
				});
			});
		}
	}

	_createClass(Scope, [{
		key: 'addDeclaration',
		value: function addDeclaration(node, isBlockDeclaration, isVar) {
			var _this2 = this;

			if (!isBlockDeclaration && this.isBlockScope) {
				// it's a `var` or function node, and this
				// is a block scope, so we need to go up
				this.parent.addDeclaration(node, isBlockDeclaration, isVar);
			} else if (node.id) {
				extractNames(node.id).forEach(function (name) {
					_this2.declarations[name] = true;
				});
			}
		}
	}, {
		key: 'contains',
		value: function contains(name) {
			return this.declarations[name] || (this.parent ? this.parent.contains(name) : false);
		}
	}]);

	return Scope;
}();

function attachScopes(ast) {
	var propertyName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'scope';

	var scope = new Scope();

	(0, _estreeWalker.walk)(ast, {
		enter: function enter(node, parent) {
			// function foo () {...}
			// class Foo {...}
			if (/(Function|Class)Declaration/.test(node.type)) {
				scope.addDeclaration(node, false, false);
			}

			// var foo = 1
			if (node.type === 'VariableDeclaration') {
				var isBlockDeclaration = blockDeclarations[node.kind];

				node.declarations.forEach(function (declaration) {
					scope.addDeclaration(declaration, isBlockDeclaration, true);
				});
			}

			var newScope = void 0;

			// create new function scope
			if (/Function/.test(node.type)) {
				newScope = new Scope({
					parent: scope,
					block: false,
					params: node.params
				});

				// named function expressions - the name is considered
				// part of the function's scope
				if (node.type === 'FunctionExpression' && node.id) {
					newScope.addDeclaration(node, false, false);
				}
			}

			// create new block scope
			if (node.type === 'BlockStatement' && !/Function/.test(parent.type)) {
				newScope = new Scope({
					parent: scope,
					block: true
				});
			}

			// catch clause has its own block scope
			if (node.type === 'CatchClause') {
				newScope = new Scope({
					parent: scope,
					params: [node.param],
					block: true
				});
			}

			if (newScope) {
				Object.defineProperty(node, propertyName, {
					value: newScope,
					configurable: true
				});

				scope = newScope;
			}
		},
		leave: function leave(node) {
			if (node[propertyName]) scope = scope.parent;
		}
	});

	return scope;
}