'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = dataToNamedExports;

var _makeLegalIdentifier = require('./makeLegalIdentifier');

var _makeLegalIdentifier2 = _interopRequireDefault(_makeLegalIdentifier);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function serializeArray(arr, indent, baseIndent) {
	var output = '[';
	var separator = indent ? '\n' + baseIndent + indent : '';
	for (var i = 0; i < arr.length; i++) {
		var key = arr[i];
		output += '' + (i > 0 ? ',' : '') + separator + serialize(key, indent, baseIndent + indent);
	}
	return output + ((indent ? '\n' + baseIndent : '') + ']');
}

function serializeObject(obj, indent, baseIndent) {
	var output = '{';
	var separator = indent ? '\n' + baseIndent + indent : '';
	var keys = Object.keys(obj);
	for (var i = 0; i < keys.length; i++) {
		var key = keys[i];
		var stringKey = (0, _makeLegalIdentifier2.default)(key) === key ? key : JSON.stringify(key);
		output += '' + (i > 0 ? ',' : '') + separator + stringKey + ':' + (indent ? ' ' : '') + serialize(obj[key], indent, baseIndent + indent);
	}
	return output + ((indent ? '\n' + baseIndent : '') + '}');
}

function serialize(obj, indent, baseIndent) {
	if (obj === Infinity) return 'Infinity';
	if (obj instanceof Date) return 'new Date(' + obj.getTime() + ')';
	if (obj instanceof RegExp) return obj.toString();
	if (typeof obj === 'number' && isNaN(obj)) return 'NaN';
	if (Array.isArray(obj)) return serializeArray(obj, indent, baseIndent);
	if (obj === null) return 'null';
	if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object') return serializeObject(obj, indent, baseIndent);
	return JSON.stringify(obj);
}

// convert data object into separate named exports (and default)
function dataToNamedExports(data) {
	var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

	var t = options.compact ? '' : 'indent' in options ? options.indent : '\t';
	var _ = options.compact ? '' : ' ';
	var n = options.compact ? '' : '\n';
	var declarationType = options.preferConst ? 'const' : 'var';

	if (options.namedExports === false || (typeof data === 'undefined' ? 'undefined' : _typeof(data)) !== 'object' || Array.isArray(data) || data === null) return 'export default' + _ + serialize(data, options.compact ? null : t, '') + ';';

	var namedExportCode = '';
	var defaultExportRows = [];
	var dataKeys = Object.keys(data);
	for (var i = 0; i < dataKeys.length; i++) {
		var key = dataKeys[i];
		if (key === (0, _makeLegalIdentifier2.default)(key)) {
			if (options.objectShorthand) defaultExportRows.push(key);else defaultExportRows.push(key + ':' + _ + key);
			namedExportCode += 'export ' + declarationType + ' ' + key + _ + '=' + _ + serialize(data[key], options.compact ? null : t, '') + ';' + n;
		} else {
			defaultExportRows.push(JSON.stringify(key) + ': ' + serialize(data[key], options.compact ? null : t, ''));
		}
	}
	return namedExportCode + ('export default' + _ + '{' + n + t + defaultExportRows.join(',' + n + t) + n + '};' + n);
}